@@ -35,6 +35,7 @@ import multiprocessing
 # Import Salt modules
 import salt.cache
 import salt.loader
+import salt.client
 import salt.output
 import salt.version
 import salt.utils.jid
@@ -44,17 +45,20 @@ from salt.minion import SMinion
 from salt.cli.batch import Batch
 import salt.utils.stringutils
 import salt.defaults.exitcodes
-from salt.exceptions import SaltSystemExit
+from salt.exceptions import SaltSystemExit, SaltInvocationError
 from salt.defaults import DEFAULT_TARGET_DELIM
 
 import salt.utils.napalm
 import salt.utils.dictupdate
 
 try:
-    from salt.utils.platform import is_proxy  # pylint: disable=unused-import
+    import salt.utils.platform
     from salt.utils.args import clean_kwargs
+
+    OLD_SALT = False
 except ImportError:
-    from salt.utils import is_proxy  # pylint: disable=unused-import
+    OLD_SALT = True
+    import salt.utils
     from salt.utils import clean_kwargs
 
 try:
@@ -95,12 +99,15 @@ def _is_proxy():
 
 
 # Same rationale as above, for any other Proxy type.
-is_proxy = _is_proxy
+if not OLD_SALT:
+    salt.utils.platform.is_proxy = _is_proxy
+else:
+    salt.utils.is_proxy = _is_proxy
 
 
 def _salt_call_and_return(
     minion_id,
-    function,
+    salt_function,
     ret_queue,
     unreachable_devices,
     failed_devices,
@@ -114,7 +121,7 @@ def _salt_call_and_return(
     opts['jid'] = jid
     ret, retcode = salt_call(
         minion_id,
-        function,
+        salt_function,
         unreachable_devices=unreachable_devices,
         failed_devices=failed_devices,
         **opts
@@ -123,7 +130,7 @@ def _salt_call_and_return(
         __salt__['event.send'](
             'proxy/runner/{jid}/ret/{minion_id}'.format(minion_id=minion_id, jid=jid),
             {
-                'fun': function,
+                'fun': salt_function,
                 'fun_args': arg,
                 'id': minion_id,
                 'jid': jid,
@@ -161,7 +168,7 @@ def _existing_proxy_cli_batch(
     batch_stop_queue.put(cumulative_retcode)
 
 
-def _receive_replies_async(ret_queue, progress_bar):
+def _receive_replies_async(ret_queue, done_queue, progress_bar):
     '''
     '''
     count = 0
@@ -181,9 +188,10 @@ def _receive_replies_async(ret_queue, progress_bar):
             salt.utils.stringutils.print_cli(out_fmt)
         if progress_bar:
             progress_bar.update(count)
+    done_queue.put(_SENTINEL)
 
 
-def _receive_replies_sync(ret_queue, static_queue, progress_bar):
+def _receive_replies_sync(ret_queue, static_queue, done_queue, progress_bar):
     '''
     '''
     count = 0
@@ -196,6 +204,28 @@ def _receive_replies_sync(ret_queue, static_queue, progress_bar):
             break
         if progress_bar:
             progress_bar.update(count)
+    done_queue.put(_SENTINEL)
+
+
+class NoPingBatch(Batch):
+    '''
+    Similar to the native Salt Batch. but without issuing test.ping to ensure
+    that the Minions are up and running.
+    '''
+
+    def __init__(
+        self, opts, eauth=None, quiet=False, parser=None
+    ):  # pylint: disable=super-init-not-called
+        self.opts = opts
+        self.eauth = eauth if eauth else {}
+        self.pub_kwargs = eauth if eauth else {}
+        self.quiet = quiet
+        self.local = salt.client.get_local_client(opts['conf_file'])
+        self.minions, self.ping_gen, self.down_minions = self.__gather_minions()
+        self.options = parser
+
+    def __gather_minions(self):
+        return self.opts['tgt'], [], []
 
 
 # The SProxyMinion class is back-ported from Salt 2019.2.0 (to be released soon)
@@ -236,15 +266,9 @@ class SProxyMinion(SMinion):
 
             salt '*' sys.reload_modules
         '''
-        cached_grains = None
-        if self.opts.get('proxy_use_cached_grains', True):
-            cached_grains = self.opts.pop('proxy_cached_grains', None)
-
-        if not cached_grains and self.opts.get('proxy_preload_grains', True):
+        if self.opts.get('proxy_preload_grains', True):
             loaded_grains = salt.loader.grains(self.opts)
             self.opts['grains'].update(loaded_grains)
-        elif cached_grains:
-            self.opts['grains'].update(cached_grains)
 
         if (
             self.opts['roster_opts']
@@ -258,21 +282,25 @@ class SProxyMinion(SMinion):
                 self.opts['roster_opts']['grains'], self.opts['grains']
             )
 
-        cached_pillar = None
-        if self.opts.get('proxy_use_cached_pillar', True):
-            cached_pillar = self.opts.pop('proxy_cached_pillar', None)
-        if not cached_pillar and self.opts.get('proxy_load_pillar', True):
+        cached_grains = None
+        if self.opts.get('proxy_use_cached_grains', True):
+            cached_grains = self.opts.pop('proxy_cached_grains', None)
+
+        initial_grains = copy.deepcopy(self.opts['grains'])
+        if cached_grains:
+            # Merging the collected Grains into the cached Grains, but only for
+            # the initial Pillar compilation, to ensure we only do so to avoid
+            # any processing errors.
+            initial_grains = salt.utils.dictupdate.merge(cached_grains, initial_grains)
+
+        if self.opts.get('proxy_load_pillar', True):
             self.opts['pillar'] = salt.pillar.get_pillar(
                 self.opts,
-                self.opts['grains'],
+                initial_grains,
                 self.opts['id'],
                 saltenv=self.opts['saltenv'],
                 pillarenv=self.opts.get('pillarenv'),
             ).compile_pillar()
-        elif cached_pillar:
-            self.opts['pillar'] = salt.utils.dictupdate.merge(
-                cached_pillar, self.opts['pillar']
-            )
 
         if self.opts['roster_opts'] and self.opts.get('proxy_merge_roster_opts', True):
             if 'proxy' not in self.opts['pillar']:
@@ -280,7 +308,8 @@ class SProxyMinion(SMinion):
             self.opts['pillar']['proxy'] = salt.utils.dictupdate.merge(
                 self.opts['pillar']['proxy'], self.opts['roster_opts']
             )
-            self.opts['pillar']['proxy'].pop('name', None)
+            self.opts['pillar']['proxy'].pop('grains', None)
+            self.opts['pillar']['proxy'].pop('pillar', None)
 
         if self.opts.get('preload_targeting', False) or self.opts.get(
             'invasive_targeting', False
@@ -310,17 +339,24 @@ class SProxyMinion(SMinion):
             raise SaltSystemExit(code=salt.defaults.exitcodes.EX_GENERIC, msg=errmsg)
 
         if 'proxy' not in self.opts:
-            self.opts['proxy'] = self.opts['pillar']['proxy']
+            self.opts['proxy'] = {}
+        if 'proxy' not in self.opts['pillar']:
+            self.opts['pillar']['proxy'] = {}
+        self.opts['proxy'] = salt.utils.dictupdate.merge(
+            self.opts['proxy'], self.opts['pillar']['proxy']
+        )
 
         # Then load the proxy module
+        fq_proxyname = self.opts['proxy']['proxytype']
         self.utils = salt.loader.utils(self.opts)
-        self.proxy = salt.loader.proxy(self.opts, utils=self.utils)
+        self.proxy = salt.loader.proxy(
+            self.opts, utils=self.utils, whitelist=[fq_proxyname]
+        )
         self.functions = salt.loader.minion_mods(
             self.opts, utils=self.utils, notify=False, proxy=self.proxy
         )
         self.functions.pack['__grains__'] = copy.deepcopy(self.opts['grains'])
 
-        fq_proxyname = self.opts['proxy']['proxytype']
         self.functions.pack['__proxy__'] = self.proxy
         self.proxy.pack['__salt__'] = self.functions
         self.proxy.pack['__pillar__'] = self.opts['pillar']
@@ -370,7 +406,7 @@ class SProxyMinion(SMinion):
                 if self.unreachable_devices is not None:
                     self.unreachable_devices.append(self.opts['id'])
                 raise
-            if not cached_grains and self.opts.get('proxy_load_grains', True):
+            if self.opts.get('proxy_load_grains', True):
                 # When the Grains are loaded from the cache, no need to re-load them
                 # again.
 
@@ -382,7 +418,17 @@ class SProxyMinion(SMinion):
                 # may contain other grains from different sources, e.g., roster.
                 loaded_grains = salt.loader.grains(self.opts, proxy=self.proxy)
                 self.opts['grains'] = salt.utils.dictupdate.merge(grains, loaded_grains)
+            if self.opts.get('proxy_load_pillar', True):
+                self.opts['pillar'] = salt.pillar.get_pillar(
+                    self.opts,
+                    self.opts['grains'],
+                    self.opts['id'],
+                    saltenv=self.opts['saltenv'],
+                    pillarenv=self.opts.get('pillarenv'),
+                ).compile_pillar()
+            self.functions.pack['__opts__'] = self.opts
             self.functions.pack['__grains__'] = copy.deepcopy(self.opts['grains'])
+            self.functions.pack['__pillar__'] = copy.deepcopy(self.opts['pillar'])
         self.grains_cache = copy.deepcopy(self.opts['grains'])
 
         if self.opts.get('invasive_targeting', False):
@@ -402,6 +448,14 @@ class SProxyMinion(SMinion):
                 proxy_shut_fn(self.opts)
                 return
 
+        self.module_executors = self.proxy.get(
+            '{0}.module_executors'.format(fq_proxyname), lambda: []
+        )() or self.opts.get('module_executors', [])
+        if self.module_executors:
+            self.executors = salt.loader.executors(
+                self.opts, self.functions, proxy=self.proxy
+            )
+
         # Late load the Returners, as they might need Grains, which may not be
         # properly or completely loaded before this.
         self.returners = None
@@ -432,7 +486,7 @@ class StandaloneProxy(SProxyMinion):
 
 def salt_call(
     minion_id,
-    function=None,
+    salt_function=None,
     unreachable_devices=None,
     failed_devices=None,
     with_grains=True,
@@ -441,8 +495,8 @@ def salt_call(
     preload_pillar=True,
     default_grains=None,
     default_pillar=None,
-    cache_grains=False,
-    cache_pillar=False,
+    cache_grains=True,
+    cache_pillar=True,
     use_cached_grains=True,
     use_cached_pillar=True,
     use_existing_proxy=False,
@@ -469,7 +523,7 @@ def salt_call(
     minion_id:
         The ID of the Minion to compile Pillar data for.
 
-    function
+    salt_function
         The name of the Salt function to invoke.
 
     preload_grains: ``True``
@@ -506,20 +560,12 @@ def salt_call(
         Use cached Grains whenever possible. If unable to gather cached data,
         it falls back to collecting Grains.
 
-    cache_pillar: ``False``
+    cache_pillar: ``True``
         Cache the compiled Pillar data before returning.
 
-        .. warning::
-            This option may be dangerous when targeting a device that already
-            has a Proxy Minion associated, however recommended otherwise.
-
-    cache_grains: ``False``
+    cache_grains: ``True``
         Cache the collected Grains before returning.
 
-        .. warning::
-            This option may be dangerous when targeting a device that already
-            has a Proxy Minion associated, however recommended otherwise.
-
     use_existing_proxy: ``False``
         Use the existing Proxy Minions when they are available (say on an
         already running Master).
@@ -559,8 +605,6 @@ def salt_call(
         opts['saltenv'] = 'base'
     if not default_grains:
         default_grains = {}
-    if use_cached_grains or use_cached_pillar:
-        minion_cache = __salt__['cache.fetch']('minions/{}'.format(minion_id), 'data')
     opts['grains'] = default_grains
     if not default_pillar:
         default_pillar = {}
@@ -573,14 +617,13 @@ def salt_call(
     opts['proxy_cache_pillar'] = cache_pillar
     opts['preload_targeting'] = preload_targeting
     opts['invasive_targeting'] = invasive_targeting
-    opts['proxy_use_cached_grains'] = use_cached_grains
     opts['proxy_no_connect'] = no_connect
     opts['proxy_test_ping'] = test_ping
+    opts['proxy_use_cached_grains'] = use_cached_grains
     if use_cached_grains:
-        opts['proxy_cached_grains'] = minion_cache.get('grains')
-    opts['proxy_use_cached_pillar'] = use_cached_pillar
-    if use_cached_pillar:
-        opts['proxy_cached_pillar'] = minion_cache.get('pillar')
+        opts['proxy_cached_grains'] = __salt__['cache.fetch'](
+            'minions/{}/data'.format(minion_id), 'grains'
+        )
     opts['roster_opts'] = roster_opts
     opts['returner'] = returner
     if not returner_kwargs:
@@ -599,11 +642,25 @@ def salt_call(
     kwargs = clean_kwargs(**kwargs)
     ret = None
     retcode = 0
+    executors = getattr(sa_proxy, 'module_executors')
     try:
-        ret = sa_proxy.functions[function](*args, **kwargs)
+        if executors:
+            for name in executors:
+                ex_name = '{}.execute'.format(name)
+                if ex_name not in sa_proxy.executors:
+                    raise SaltInvocationError(
+                        "Executor '{0}' is not available".format(name)
+                    )
+                ret = sa_proxy.executors[ex_name](
+                    opts, {'fun': salt_function}, salt_function, args, kwargs
+                )
+                if ret is not None:
+                    break
+        else:
+            ret = sa_proxy.functions[salt_function](*args, **kwargs)
         retcode = sa_proxy.functions.pack['__context__'].get('retcode', 0)
     except Exception as err:
-        log.info('Exception while running %s on %s', function, opts['id'])
+        log.info('Exception while running %s on %s', salt_function, opts['id'])
         if failed_devices is not None:
             failed_devices.append(opts['id'])
         ret = 'The minion function caused an exception: {err}'.format(
@@ -626,7 +683,7 @@ def salt_call(
             ret_data = {
                 'id': opts['id'],
                 'jid': jid,
-                'fun': function,
+                'fun': salt_function,
                 'fun_args': args,
                 'return': ret,
                 'ret_config': returner_config,
@@ -661,7 +718,7 @@ def salt_call(
 
 def execute_devices(
     minions,
-    function,
+    salt_function,
     with_grains=True,
     with_pillar=True,
     preload_grains=True,
@@ -669,15 +726,15 @@ def execute_devices(
     default_grains=None,
     default_pillar=None,
     args=(),
-    batch_size=10,
+    batch_size=None,
     batch_wait=0,
     static=False,
     tgt=None,
     tgt_type=None,
     jid=None,
     events=True,
-    cache_grains=False,
-    cache_pillar=False,
+    cache_grains=True,
+    cache_pillar=True,
     use_cached_grains=True,
     use_cached_pillar=True,
     use_existing_proxy=False,
@@ -705,7 +762,7 @@ def execute_devices(
     minions
         A list of Minion IDs to invoke ``function`` on.
 
-    function
+    salt_function
         The name of the Salt function to invoke.
 
     preload_grains: ``True``
@@ -740,7 +797,7 @@ def execute_devices(
     kwargs
         Key-value arguments to send to the Salt function.
 
-    batch_size: ``10``
+    batch_size: None
         The size of each batch to execute.
 
     static: ``False``
@@ -759,20 +816,12 @@ def execute_devices(
         Use cached Grains whenever possible. If unable to gather cached data,
         it falls back to collecting Grains.
 
-    cache_pillar: ``False``
+    cache_pillar: ``True``
         Cache the compiled Pillar data before returning.
 
-        .. warning::
-            This option may be dangerous when targeting a device that already
-            has a Proxy Minion associated, however recommended otherwise.
-
-    cache_grains: ``False``
+    cache_grains: ``True``
         Cache the collected Grains before returning.
 
-        .. warning::
-            This option may be dangerous when targeting a device that already
-            has a Proxy Minion associated, however recommended otherwise.
-
     use_existing_proxy: ``False``
         Use the existing Proxy Minions when they are available (say on an
         already running Master).
@@ -802,7 +851,7 @@ def execute_devices(
         if salt.version.__version_info__ >= (2018, 3, 0):
             jid = salt.utils.jid.gen_jid(__opts__)
         else:
-            jid = salt.utils.jid.gen_jid()
+            jid = salt.utils.jid.gen_jid()  # pylint: disable=no-value-for-parameter
     event_args = list(args[:])
     if kwargs:
         event_kwargs = {'__kwarg__': True}
@@ -840,7 +889,7 @@ def execute_devices(
         __salt__['event.send'](
             'proxy/runner/{jid}/new'.format(jid=jid),
             {
-                'fun': function,
+                'fun': salt_function,
                 'minions': minions,
                 'arg': event_args,
                 'jid': jid,
@@ -859,30 +908,46 @@ def execute_devices(
             max_value=len(minions), enable_colors=True, redirect_stdout=True
         )
     ret_queue = multiprocessing.Queue()
+    done_queue = multiprocessing.Queue()
     if not static:
         thread = threading.Thread(
-            target=_receive_replies_async, args=(ret_queue, progress_bar)
+            target=_receive_replies_async, args=(ret_queue, done_queue, progress_bar)
         )
         thread.daemon = True
         thread.start()
     else:
         static_queue = multiprocessing.Queue()
         thread = threading.Thread(
-            target=_receive_replies_sync, args=(ret_queue, static_queue, progress_bar)
+            target=_receive_replies_sync,
+            args=(ret_queue, static_queue, done_queue, progress_bar),
         )
         thread.daemon = True
         thread.start()
 
     ret = {}
-    batch_size = int(batch_size)
-    batch_count = int(len(minions) / batch_size) + (
-        1 if len(minions) % batch_size else 0
-    )
-    existing_batch_size = int(
-        math.ceil(len(existing_minions) * batch_size / float(len(minions)))
-    )
-    sproxy_batch_size = batch_size - existing_batch_size
     sproxy_minions = list(set(minions) - set(existing_minions))
+    if batch_size:
+        if '%' in str(batch_size):
+            percent = int(batch_size.replace('%', ''))
+            batch_size = len(minions) * percent / 100
+        batch_size = int(batch_size)
+        batch_count = int(len(minions) / batch_size) + (
+            1 if len(minions) % batch_size else 0
+        )
+        existing_batch_size = int(
+            math.ceil(len(existing_minions) * batch_size / float(len(minions)))
+        )
+        sproxy_batch_size = batch_size - existing_batch_size
+    else:
+        # when no explicit batch requested, we'll execute the command on the
+        # existing minions without any batching (i.e., on all the matched
+        # minions at once), while sproxy ones are executed in as many CPUs are
+        # available.
+        sproxy_batch_size = multiprocessing.cpu_count()
+        existing_batch_size = len(existing_minions)
+        batch_count = int(len(sproxy_minions) / sproxy_batch_size) + (
+            1 if len(sproxy_minions) % sproxy_batch_size else 0
+        )
     cli_batch = None
     if existing_batch_size > 0:
         # When there are existing Minions matching the target, use the native
@@ -893,14 +958,17 @@ def execute_devices(
         batch_opts['batch'] = str(existing_batch_size)
         batch_opts['tgt'] = existing_minions
         batch_opts['tgt_type'] = 'list'
-        batch_opts['fun'] = function
+        batch_opts['fun'] = salt_function
         batch_opts['arg'] = event_args
         batch_opts['batch_wait'] = batch_wait
         batch_opts['selected_target_option'] = 'list'
         batch_opts['return'] = returner
         batch_opts['ret_config'] = returner_config
         batch_opts['ret_kwargs'] = returner_kwargs
-        cli_batch = Batch(batch_opts, quiet=True)
+        if test_ping:
+            cli_batch = Batch(batch_opts, quiet=True)
+        else:
+            cli_batch = NoPingBatch(batch_opts, quiet=True)
         log.debug('Batching detected the following Minions responsive')
         log.debug(cli_batch.minions)
         if cli_batch.down_minions:
@@ -910,6 +978,14 @@ def execute_devices(
                 ', '.join(cli_batch.down_minions),
             )
             down_minions = cli_batch.down_minions
+            for minion in down_minions:
+                ret_queue.put(
+                    (
+                        {minion: 'Minion did not return. [Not connected]'},
+                        salt.defaults.exitcodes.EX_UNAVAILABLE,
+                    )
+                )
+
     log.info(
         '%d devices matched the target, executing in %d batches',
         len(minions),
@@ -975,7 +1051,7 @@ def execute_devices(
         while not sproxy_execute_queue.empty() and not stop_iteration:
             if len(sproxy_processes) >= sproxy_batch_size:
                 # Wait for the bucket to make room for another process.
-                time.sleep(0.02)
+                time.sleep(0.001)
                 continue
             minion_id, device_opts = sproxy_execute_queue.get()
             log.debug('Starting execution for %s', minion_id)
@@ -984,7 +1060,7 @@ def execute_devices(
                 name=minion_id,
                 args=(
                     minion_id,
-                    function,
+                    salt_function,
                     ret_queue,
                     unreachable_devices,
                     failed_devices,
@@ -1064,7 +1140,7 @@ def execute_devices(
 
         # Waiting for the existing proxy batch to finish.
         while batch_stop_queue.empty():
-            time.sleep(0.02)
+            time.sleep(0.001)
         batch_retcode = batch_stop_queue.get()
         retcode = max(retcode, batch_retcode)
 
@@ -1072,7 +1148,9 @@ def execute_devices(
         ret_queue.put((_SENTINEL, 0))
         # Wait a little to dequeue and print before throwing the progressbar,
         # the summary, etc.
-        time.sleep(0.02)
+        while done_queue.empty():
+            time.sleep(0.001)
+
         if progress_bar:
             progress_bar.finish()
 
@@ -1142,7 +1220,7 @@ def execute_devices(
                     {
                         'tgt': tgt,
                         'tgt_type': tgt_type,
-                        'fun': function,
+                        'fun': salt_function,
                         'fun_args': event_args,
                         'jid': jid,
                         'user': __pub_user,
@@ -1166,7 +1244,7 @@ def execute_devices(
 
 def execute(
     tgt,
-    function=None,
+    salt_function=None,
     tgt_type='glob',
     roster=None,
     preview_target=False,
@@ -1179,12 +1257,12 @@ def execute(
     default_grains=None,
     default_pillar=None,
     args=(),
-    batch_size=10,
+    batch_size=None,
     batch_wait=0,
     static=False,
     events=True,
-    cache_grains=False,
-    cache_pillar=False,
+    cache_grains=True,
+    cache_pillar=True,
     use_cached_grains=True,
     use_cached_pillar=True,
     use_existing_proxy=False,
@@ -1218,7 +1296,7 @@ def execute(
         for a list, etc. The ``tgt_list`` argument must be used accordingly,
         depending on the type of this expression.
 
-    function
+    salt_function
         The name of the Salt function to invoke.
 
     tgt_type: ``glob``
@@ -1262,7 +1340,7 @@ def execute(
     kwargs
         Key-value arguments to send to the Salt function.
 
-    batch_size: ``10``
+    batch_size: None
         The size of each batch to execute.
 
     static: ``False``
@@ -1281,20 +1359,12 @@ def execute(
         Use cached Grains whenever possible. If unable to gather cached data,
         it falls back to collecting Grains.
 
-    cache_pillar: ``False``
+    cache_pillar: ``True``
         Cache the compiled Pillar data before returning.
 
-        .. warning::
-            This option may be dangerous when targeting a device that already
-            has a Proxy Minion associated, however recommended otherwise.
-
-    cache_grains: ``False``
+    cache_grains: ``True``
         Cache the collected Grains before returning.
 
-        .. warning::
-            This option may be dangerous when targeting a device that already
-            has a Proxy Minion associated, however recommended otherwise.
-
     use_existing_proxy: ``False``
         Use the existing Proxy Minions when they are available (say on an
         already running Master).
@@ -1326,9 +1396,9 @@ def execute(
     roster = roster or __opts__.get('proxy_roster', __opts__.get('roster'))
 
     saltenv = __opts__.get('saltenv', 'base')
-    if sync_roster:
+    if sync_roster and not sync_all:
         __salt__['saltutil.sync_roster'](saltenv=saltenv)
-    if sync_modules:
+    if sync_modules and not sync_all:
         __salt__['saltutil.sync_modules'](saltenv=saltenv)
     if sync_all:
         __salt__['saltutil.sync_all'](saltenv=saltenv)
@@ -1340,6 +1410,15 @@ def execute(
         # indefinitely.
         timeout = None
 
+    if tgt_type == 'pillar_target':
+        # When using the -I option on the CLI, the tgt_type passed on is called
+        # `pillar_target`:
+        # https://github.com/saltstack/salt/blob/e9e48b7fb6a688f4f22d74a849d58c1c156563d1/salt/utils/parsers.py#L1266
+        # While if we want to use this against existing Minions, the option
+        # needs to be just `pillar`:
+        # https://github.com/saltstack/salt/blob/99385b50718d70d93fd5b83e61c0f4b3a402490c/salt/utils/minions.py#L359
+        tgt_type = 'pillar'
+
     if preload_targeting or invasive_targeting:
         _tgt = '*'
         _tgt_type = 'glob'
@@ -1401,7 +1480,9 @@ def execute(
             log.debug('Computing the target using the %s Roster', roster)
             __opts__['use_cached_grains'] = use_cached_grains
             __opts__['use_cached_pillar'] = use_cached_pillar
-            roster_modules = salt.loader.roster(__opts__, runner=__salt__)
+            roster_modules = salt.loader.roster(
+                __opts__, runner=__salt__, whitelist=[roster]
+            )
             if '.targets' not in roster:
                 roster = '{mod}.targets'.format(mod=roster)
             rtargets_roster = roster_modules[roster](_tgt, tgt_type=_tgt_type)
@@ -1423,14 +1504,14 @@ def execute(
         return 'No devices matched your target. Please review your tgt / tgt_type arguments, or the Roster data source'
     if preview_target:
         return targets
-    elif not function:
+    elif not salt_function:
         return 'Please specify a Salt function to execute.'
     jid = kwargs.get('__pub_jid')
     if not jid:
         if salt.version.__version_info__ >= (2018, 3, 0):
             jid = salt.utils.jid.gen_jid(__opts__)
         else:
-            jid = salt.utils.jid.gen_jid()
+            jid = salt.utils.jid.gen_jid()  # pylint: disable=no-value-for-parameter
     if verbose or show_jid:
         salt.utils.stringutils.print_cli('Executing job with jid {0}'.format(jid))
         salt.utils.stringutils.print_cli(
@@ -1440,7 +1521,7 @@ def execute(
         __salt__['event.send'](jid, {'minions': targets})
     return execute_devices(
         targets,
-        function,
+        salt_function,
         tgt=tgt,
         tgt_type=tgt_type,
         with_grains=with_grains,
